/* Copyright 2004,2005 Thomas "thisnukes4u" Coppi
   If you use this program, please send me an e-mail at thisnukes4u@gmail.com
   You may do whatever you wish with this program except for selling it, claiming it as your own,
   or modifying it without releasing the source code (the listed restrictions are not mutally exclusive,
   they all apply). If you do not agree with the above terms, you may optionally distribute and use 
   this software under the GNU GPL, the terms of which are included in the COPYING file */

#include "rssfeed.h"
#include "conf.h"
#include "gui.h"
#include "error.h"
#include "config.h"

#ifndef NOGNET
#include <gnet.h>
#include <conn-http.h>
#endif

/* FIXME Gnet doesn't have a prototype for gnet_http_get, so we have to put it in ourselves... FIXME */
gboolean gnet_http_get(const gchar *url,gchar **buffer,gsize *length,guint *response);

extern FILE *logfp;

/* helper functions */
static int parse_channel(xmlDocPtr doc, xmlNodePtr cur, RssFeed *feed);
static int parse_item(xmlDocPtr doc, xmlNodePtr cur, RssFeed *feed);
static xmlDocPtr gfeed_parse_file(char *location);

/* Loads the feed and parses 
   each section using the
   apporpriate function */
int parse(RssFeed *feed)
{
	  xmlDocPtr doc;
	  xmlNodePtr cur;

	  feed->currnum = 0;
	  printf("Please wait, loading RSS/RDF file %s...\n",feed->location);

	  doc = gfeed_parse_file(feed->location);
	  
	  if(!doc) {
	  	fprintf(stderr,"Document not parsed successfully. \n");
	    	return -1;	
	  }
	  
	  cur = xmlDocGetRootElement(doc);
	  
	  if(!cur) {
	    	fprintf(stderr,"empty document\n");
	    	xmlFreeDoc(doc);
	    	return -1;
	  }
	  
	  /* check for a correctly formatted feed(at least the root node...)*/
	  if(xmlStrcmp(cur->name, (const xmlChar *)"rss") && xmlStrcmp(cur->name, (const xmlChar *)"rdf") &&
			xmlStrcmp(cur->name, (const xmlChar *)"RSS") && xmlStrcmp(cur->name, (const xmlChar *)"RDF"))
	    			gfeed_error(logfp, FEED_TYPE);	  
	  
	  cur = cur->xmlChildrenNode;
	  
	  while(cur != NULL) {
		if((!xmlStrcmp(cur->name, (const xmlChar *)"channel"))){
	      		parse_channel(doc, cur, feed);
		}
	    	if((!xmlStrcmp(cur->name, (const xmlChar *)"item"))){
	      		parse_item(doc, cur, feed);
	     		feed->currnum++;
	    	}
	    	cur = cur->next;
	  }
	  	  
	  printf("Finished Loading RSS/RDF file.\n");
	  xmlFreeDoc(doc);
	  return 0;
}

/* Parses a channel */
static int parse_channel(xmlDocPtr doc, xmlNodePtr cur, RssFeed *feed)
{
	  xmlChar *key;

	  cur = cur->xmlChildrenNode;

	  while(cur != NULL) {
	    	if((!xmlStrcmp(cur->name, (const xmlChar *)"title"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->channel_title = (char *)key;
	    	}
	   	else if((!xmlStrcmp(cur->name, (const xmlChar *)"link"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->channel_link = (char *)key;
	    	}
	   	else if((!xmlStrcmp(cur->name, (const xmlChar *)"description"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->channel_desc = (char *)key;
	    	}
	    	else if((!xmlStrcmp(cur->name, (const xmlChar *)"language"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->channel_lang = (char *)key;
	    	}
	    	else if((!xmlStrcmp(cur->name, (const xmlChar *)"item"))) {
	      		parse_item(doc, cur, feed);
	      		feed->currnum++;
	    	}
	    cur = cur->next;
	  }
	  
	  return 0;
}

/* Parses an item */
static int parse_item(xmlDocPtr doc, xmlNodePtr cur, RssFeed *feed)
{
	  xmlChar *key;

	  cur = cur->xmlChildrenNode;

	  while(cur != NULL) {
	    	if((!xmlStrcmp(cur->name, (const xmlChar *)"title"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->item_title[feed->currnum] = (char *)key;
	    	}
	    	else if((!xmlStrcmp(cur->name, (const xmlChar *)"link"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	     		feed->item_link[feed->currnum] = (char *)key;
	    	}
	    	else if((!xmlStrcmp(cur->name, (const xmlChar *)"description"))) {
	      		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
	      		feed->item_desc[feed->currnum] = (char *)key;
		}
		else if((!xmlStrcmp(cur->name, (const xmlChar *)"pubDate"))) {
			key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
			feed->item_date[feed->currnum] = (char *)key;
		}
		else if((!xmlStrcmp(cur->name, (const xmlChar *)"date"))) {
			key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 0);
			feed->item_date[feed->currnum] = (char *)key;
		}
					
	    cur = cur->next;
	  }
		  
	  return 0;
}

/* Setup the new RssFeed */
void new_rssfeed(char *location, RssFeed *feed)
{
	int i;

	feed->currnum = 0;	
	feed->location = location;

	feed->channel_desc = NULL;	
	feed->channel_lang = NULL;
	feed->channel_link = NULL;
	feed->channel_title = NULL;
	
	for(i=0;i<MAX_ITEMS;i++) {
		feed->item_title[i] = NULL;
		feed->item_link[i] = NULL;
		feed->item_desc[i] = NULL;
		feed->item_date[i] = NULL;
	}
}

/* Free the allocated memory in the
   feed for either re-parsing or
   ending the program */
void free_rssfeed(RssFeed *feed)
{
	int i;
	
	if(!feed->channel_title);
		else free(feed->channel_title);
	if(!feed->channel_lang);
		else free(feed->channel_lang);
	if(!feed->channel_desc);
		else free(feed->channel_desc);
	if(!feed->channel_link);
		free(feed->channel_link);
		
	feed->channel_title = feed->channel_lang = feed->channel_desc = feed->channel_link = NULL;

	for(i=0;i<feed->currnum;i++) {
		if(!feed->item_title[i]);
			else free(feed->item_title[i]);
		if(!feed->item_link[i]);
			else free(feed->item_link[i]);
		if(feed->item_desc[i] == "" || !feed->item_desc[i]);
			else free(feed->item_desc[i]);
		if(!feed->item_date[i]);
			else free(feed->item_date[i]);
			
		feed->item_title[i] = feed->item_link[i] = feed->item_desc[i] = feed->item_date[i] = NULL;
	}
}

static xmlDocPtr gfeed_parse_file(char *location)
{
#ifndef NOGNET
	char *data = NULL;
	gsize length;

	if(!strncmp(location,"file://",7)) /* if we have a file uri, we can't use gnet to parse it */
		return xmlParseFile(location + 7);
	else {
		gnet_http_get(location,&data,&length,NULL);
		if(!data) gfeed_error(logfp, RES_UNAV);
		return xmlReadMemory(data,length,NULL,NULL,0);
	}
	
	/* should never get here... */
	return NULL;
#endif
#ifdef NOGNET
	return xmlParseFile(location);
#endif	
}

